#!/usr/bin/env python3

from ase.io import read
import argparse, os
import numpy as np
from ase.build import make_supercell
from util import (
    structure_parsing,
    encapsulated_ase,
    cp2k_input_parsing
    )


def supercell(atom, x, y, z):
    P = [ [x, 0, 0], [0, y, 0], [0, 0, z] ]
    super_atom = make_supercell(atom, P)
    return super_atom


def parse_cell(s):
    return [float(x) for x in s.replace(',', ' ').split()]


def super_cell(s):
    super_cell = [int(x) for x in s.replace(',', ' ').split()]
    leng = len(super_cell)
    if leng == 2:
        super_cell.append(1)
    elif leng == 3:
        pass
    else:
        print('wrong super cell size')
        exit(1)

    return super_cell


def parse_argument():
    parser = argparse.ArgumentParser(description='make a supercell')

    parser.add_argument('input_file_name', type=str, help='input file name')
    parser.add_argument('super', type=super_cell, help='super cell size, a,b,c')
    parser.add_argument('-o', type=str, help='output file name, default is "super.cif"', default='super.cif')
    parser.add_argument('--cp2k_input_file', type=str, help='input file name of cp2k, default is "input.inp"', default='input.inp')
    parser.add_argument('--coord', help='coord format', action='store_true')
    parser.add_argument('--cell', type=parse_cell, help='set cell, a list of lattice, --cell x,y,z or x,y,z,a,b,c')

    return parser.parse_args()


def main():
    args = parse_argument()
    if args.input_file_name == None:
        print('give a xyz file')
        sys.exit()

    atom = encapsulated_ase.atoms_read_with_cell(args.input_file_name, cell=args.cell, coord_mode=args.coord)
    atom.set_pbc(True)
    atom.wrap()
    super_atom = supercell(atom, args.super[0], args.super[1], args.super[2])

    suffix = args.o.split('.')[-1]
    if suffix == 'data':
        super_atom.write(f'{args.o}', format='lammps-data', atom_style='atomic')
    else:
        super_atom.write(f'{args.o}')

    print(os.path.abspath(args.o))


if __name__ == '__main__':
    main()



#!/usr/bin/env python3

import numpy as np
import click
import os, glob


def match_file(patch):
    pwd = os.getcwd()
    match_file = glob.glob(os.path.join(pwd, patch))

    return match_file


@click.command(name='data')
@click.argument('filename', nargs=-1, type=click.Path(exists=True))
@click.option('--nor', help='normalized data', is_flag=True)
@click.option('--gaus', type=int, help='gaussian filter 1d data', default=0)
@click.option('--fold', help='fold and average', is_flag=True)
@click.option('--err', help='error bar of data', is_flag=True)
@click.option('--int_cn', type=float, nargs=2, help='integrate gofr data to coordination number', default=None)
def main(filename, nor, gaus, fold, err, int_cn):
    """
    trade data file with different method
    """

    for data_file in filename:
        data = np.loadtxt(data_file)
        data_name = data_file.split(os.sep)[-1]
        if len(data.shape) == 1:
            data = np.hstack((np.arange(data.shape[0]).reshape(-1, 1), data.reshape(-1, 1)))
        data_range = data.shape[1]
        if nor:
            nor_data = data.copy()
            for i in range(1, data_range):
                nor_data[:, i] = data[:, i] / np.max(data[:, i])
            np.savetxt(f'./nor_{data_name}', nor_data, fmt='%.2e', delimiter='\t')
        elif gaus > 0:
            from scipy.ndimage import gaussian_filter1d
            gaus_data = data.copy()
            for i in range(1, data_range):
                gaus_data[:, i] = gaussian_filter1d(data[:, i], gaus)
            np.savetxt(f"./gaus_{data_name}", gaus_data, fmt='%.2e', delimiter='\t')
        elif fold:
            data_shape = data.shape
            mid = int(data_shape[0] // 2)
            left_data = data[:mid, 1:]
            right_data = data[mid:, 1:][::-1]
            if data_shape[0] % 2 != 0:
                left_data = np.vstack((left_data, right_data[-1]))
                mid += 1
            fold_data = (left_data + right_data) / 2
            fold_data = np.hstack((data[:mid, 0].reshape(-1, 1), fold_data))
            np.savetxt(f"./fold_{data_name}", fold_data, fmt='%.2e', delimiter='\t')
        elif err:
            err_data = data.copy()
            std_dev_list = []
            for i in range(1, data_range):
                split_array = np.array_split(err_data[:, i], 5)
                averages = np.array([np.mean(part) for part in split_array])
                std_dev = averages.std(axis=0)
                std_dev_list.append(std_dev)
            np.savetxt(f"./error_{data_name}", np.vstack(std_dev_list), fmt='%.5f', delimiter='\t')
        elif int_cn is not None:
            from scipy import integrate
            rho = 32/(9.86**3)
            x = data[:, 0]
            mask = (x >= int_cn[0]) & (x <= int_cn[1])
            filtered_x = x[mask]
            filtered_y = data[:, 1][mask]
            integrate_y = 4 * np.pi * rho * np.insert(integrate.cumulative_trapezoid(filtered_y*filtered_x**2, filtered_x), 0, 0)
            np.savetxt(f"int_cn_{data_name}", np.hstack((filtered_x.reshape(-1, 1), integrate_y.reshape(-1, 1))), fmt='%.5f', delimiter='\t')



        print(f"================ processing of {data_name} is done ===================")


if __name__ == '__main__':
    main()



#!/usr/bin/env python3

import argparse, multiprocessing, os
import numpy as np
from util import (
    structure_parsing,
    numpy_geo,
    os_operation,
    cp2k_input_parsing,
    )


def hb_count(chunk, index, cell, filename, hb_distance=3.5, hb_angle=35):
    groups = structure_parsing.chunk_to_groups(chunk)
    groups_hb_list = []
    coefficients = numpy_geo.cell_to_wrap_coefficients(cell)
    for group in groups:
        group_hb_array = np.zeros((3, 1))
        present_index = index
        o_present = group[present_index].split()
        if o_present[0] == 'O':
            o_present = np.array(o_present[1:], dtype=np.float64)
            group_hb_array[2, 0] += 1
            for other_index in range(2, len(group)):
                o_other = group[other_index].split()
                if o_other[0] == 'O':
                    o_other = np.array(o_other[1:], dtype=np.float64)
                    oo_distance, o_other = numpy_geo.unwrap(o_present, o_other, coefficients, max=0)
                    if oo_distance < hb_distance and oo_distance > 1:
                        _, o_present_h1 = numpy_geo.unwrap(o_present, np.array(group[present_index+1].split()[1:], dtype=np.float64), coefficients)
                        _, o_present_h2 = numpy_geo.unwrap(o_present, np.array(group[present_index+2].split()[1:], dtype=np.float64), coefficients)
                        _, o_other_h1 = numpy_geo.unwrap(o_other, np.array(group[other_index+1].split()[1:], dtype=np.float64), coefficients)
                        _, o_other_h2 = numpy_geo.unwrap(o_other, np.array(group[other_index+2].split()[1:], dtype=np.float64), coefficients)

                        o_present_o_other_h1_angle = numpy_geo.vector_vector_angle(o_present-o_other, o_other_h1-o_other)
                        o_present_o_other_h2_angle = numpy_geo.vector_vector_angle(o_present-o_other, o_other_h2-o_other)
                        if o_present_o_other_h1_angle < hb_angle or o_present_o_other_h2_angle < hb_angle:
                            group_hb_array[0, 0] += 1
                        o_other_o_present_h1_angle = numpy_geo.vector_vector_angle(o_other-o_present, o_present_h1-o_present)
                        o_other_o_present_h2_angle = numpy_geo.vector_vector_angle(o_other-o_present, o_present_h2-o_present)
                        if o_other_o_present_h1_angle < hb_angle or o_other_o_present_h2_angle < hb_angle:
                            group_hb_array[1, 0] += 1
        groups_hb_list.append(group_hb_array)
    groups_hb_array = np.vstack(groups_hb_list)
    group_hb_acc_array = np.sum(groups_hb_array[0::3], axis=0).reshape(1, -1)
    group_hb_don_array = np.sum(groups_hb_array[1::3], axis=0).reshape(1, -1)
    group_hb_num_array = np.sum(groups_hb_array[2::3], axis=0).reshape(1, -1)
    group_hb_array = np.vstack([group_hb_acc_array, group_hb_don_array, group_hb_num_array])
    np.save(filename, group_hb_array)


def parse_data(s):
    return [float(x) for x in s.replace(',', ' ').split()]

def parse_argument():
    parser = argparse.ArgumentParser(description="analysis an O atom's hydrogen bond in water")
    parser.add_argument('index', type=int, help='index of target atom in coord.xyz, or all of hb distribution on z')
    parser.add_argument('input_file_name', type=str, nargs='?', help='input file name', default=os_operation.default_file_name('wraped.xyz', last=True))
    parser.add_argument('--cp2k_input_file', type=str, help='input file name of cp2k, default is "input.inp"', default='input.inp')
    parser.add_argument('--cell', type=parse_data, help='set cell, a list of lattice, --cell x,y,z or x,y,z,a,b,c')
    parser.add_argument('--hb_param', type=parse_data, help='[hb_distance, hb_angle], default is [3.5, 35]', default=[3.5, 35])
    parser.add_argument('--process', type=int, help='paralle process number default is 28', default=28)
    parser.add_argument('--temp', help='keep temp file', action='store_false')

    return parser.parse_args()

def main():
    args = parse_argument()
    output = f'./hb_{args.index}.dat'
    cell = cp2k_input_parsing.get_cell(args.cp2k_input_file, args.cell)
    chunks = structure_parsing.xyz_to_chunks(args.input_file_name, args.process)
    temp_dir = f'{os.environ.get("TEMP_DIR")}/{os.getpid()}'
    os_operation.make_temp_dir(temp_dir, delete=args.temp)

    for index, chunk in enumerate(chunks):
        t = multiprocessing.Process(target=hb_count, args=[chunk, args.index, cell, f'{temp_dir}/chunk_{index}.temp'])
        t.start()

    for t in multiprocessing.active_children():
        t.join()

    chunks_array_list = []
    for i in range(len(chunks)):
        chunk_array = np.load(f'{temp_dir}/chunk_{i}.temp.npy')
        chunks_array_list.append(chunk_array)
    chunks_array = np.vstack(chunks_array_list)
    chunks_array = np.mean(chunks_array, axis=1)

    with open(output, 'w') as f:
        f.write(f"# {args.index}\n")
        f.write(f"accepter : {chunks_array[0]:.2f}\n")
        f.write(f"donor    : {chunks_array[1]:.2f}\n")
        f.write(f"total    : {chunks_array[0]+chunks_array[1]:.2f}\n")
    print(f"# {args.index}")
    print(f"accepter : {chunks_array[0]:.2f}")
    print(f"donor    : {chunks_array[1]:.2f}")
    print(f"total    : {chunks_array[0]+chunks_array[1]:.2f}")


if __name__ == '__main__':
    main()



import click
from mdkits.build_cli import build_cli
from mdkits.cli import (
    convert,
    wrap,
    extract,
    data,
    plot,
    density,
    cube,
    pdos,
)


@click.group(name='cli')
@click.pass_context
@click.version_option()
def cli(ctx):
    """kits for md or dft"""
    pass


cli.add_command(convert.main)
cli.add_command(wrap.main)
cli.add_command(extract.main)
cli.add_command(data.main)
cli.add_command(plot.main)
cli.add_command(density.main)
cli.add_command(cube.main)
cli.add_command(pdos.main)
cli.add_command(build_cli.cli_build)


if __name__ == '__main__':
    cli()


import click
#from mdkits.cli.build import (
#    build_bulk,
#    build_surface,
#    adsorbate,
#)
from mdkits.build_cli import (
    build_bulk,
    build_surface,
    adsorbate,
    build_solution,
)


@click.group(name='build')
@click.pass_context
def cli_build(ctx):
    """kits for building"""
    pass


cli_build.add_command(build_bulk.main)
cli_build.add_command(build_surface.main)
cli_build.add_command(adsorbate.main)
cli_build.add_command(build_solution.main)

if __name__ == '__main__':
    cli_build()


import argparse


def parse_cell(s):
    return [float(x) for x in s.replace(',', ' ').split()]


def parse_argument():
    parser = argparse.ArgumentParser(description='generate packmol input file with give parameter')
    parser.add_argument('--size', type=int, help='water size default is 30', default=30)
    parser.add_argument('--cell', type=parse_cell, help='input box size(a,b,c)')
    parser.add_argument('--addwat', type=int, help='add some additional water, default is 0', default=0)
    parser.add_argument('--ioncon', type=float, help='concentration of sol box, default is 0.0', default=0.0)
    parser.add_argument('--tolerance', type=float, help='tolerance of packmol, default is 2.5', default=2.5)
    parser.add_argument('--watpath', type=str, help='water xyz file path', default='C:\\home\\.can\\temp\\packmol\\default\\water.xyz')
    parser.add_argument('--ionpath', type=str, help='ion xyz file path')
    parser.add_argument('-o', type=str, help='output file name, default is "input.pm"', default='input.pm')
    parser.add_argument('--output', type=str, help='output file name of packmol, default is "solbox.xyz"', default='solbox.xyz')

    return parser.parse_args()


def get_water_number():
    water_number = water_volume / water_size

    return int(round(water_number, 0))


def get_ion_number(concentration):
    ion_number = ( (concentration * avogadro) / 1e+27 ) * water_volume

    return int(round(ion_number, 0))


def main():
    global water_volume, water_size, avogadro
    args = parse_argument()
    water_volume = args.cell[0] * args.cell[1] * args.cell[2]
    water_size = args.size
    avogadro = 6.02214179e+23
    water_number = get_water_number() + args.addwat
    ion_number = get_ion_number(args.ioncon)

    if ion_number == 0:
        packmol_input_str = f"""
        tolerance {args.tolerance}
        filetype xyz
        output {args.output}
        pbc {args.cell[3]} {args.cell[4]} {args.cell[5]}
        structure {args.watpath}
          number {water_number}
          inside box 2. 2. 2. {args.cell[0]-2} {args.cell[1]-2} {args.cell[2]-2}
        end structure
        """
    else:
        packmol_input_str = f"""
        tolerance {args.tolerance}
        filetype xyz
        output {args.output}
        pbc {args.cell[3]} {args.cell[4]} {args.cell[5]}
        structure {args.watpath}
          number {water_number}
          inside box 2. 2. 2. {args.cell[0]-2} {args.cell[1]-2} {args.cell[2]-2}
        end structure
        structure {args.ionpath}
          number {ion_number}
          inside box 2. 2. 2. {args.cell[0]-2} {args.cell[1]-2} {args.cell[2]-2}
        end structure
        """

    with open(args.o, 'w') as f:
        f.write(packmol_input_str)


if __name__ == '__main__':
    main()



"""
filename: structure_parsing.py
function: prase file with .xyz structue
"""


import numpy as np


def xyz_to_chunks(filename, thread):
    """
    function: split .xyz file to chunks to parallel analysis
    """
    with open(filename, 'r') as f:
        data = f.read()

    #with open(filename, 'r') as f:
    #    atom_number = int(f.readline().split()[0]) + 2

    atom_number = int(data[:data.index('\n') + 1]) + 2
    line_data = data.split('\n')
    chunk_size = (len(line_data) / atom_number) // thread
    chunks = [line_data[i:i+int(chunk_size*atom_number)] for i in range(0, len(line_data), int(chunk_size*atom_number))]
    return chunks


def xyz_to_groups(filename, cut=0):
    """
    function: split .xyz file to groups, every groups is one single frame
    """
    with open(filename, 'r') as f:
        data = f.read()

    # read number of atoms
    end_index = data.index('\n') + 1
    natom = int(data[:end_index]) + 2

    # seqrate file with natom +2
    lines = data.split('\n')
    groups = [lines[i+cut:i+natom] for i in range(0, len(lines), natom)][:-1]
    return groups


def xyz_to_specified_list(filename, atom_range, cut=0):
    """
    function: split .xyz file to specified list, specified atoms position list
    """
    with open(filename, 'r') as f:
        data = f.read()

    # read number of atoms
    end_index = data.index('\n') + 1
    natom = int(data[:end_index]) + 2

    # seqrate file with natom +2
    lines = data.split('\n')
    if len(atom_range) > 1:
        groups = [lines[i+atom_range[0]:i+atom_range[1]] for i in range(0, len(lines), natom)][:-1]
    else:
        groups = [lines[i+atom_range[0]] for i in range(0, len(lines), natom)][:-1]
    return groups


def xyz_to_specified_array(filename, atom_range, cut=0):
    """
    function: split .xyz file to specified list, specified atoms position list
    """
    with open(filename, 'r') as f:
        data = f.read()

    # read number of atoms
    end_index = data.index('\n') + 1
    natom = int(data[:end_index]) + 2

    # seqrate file with natom +2
    lines = data.split('\n')
    if len(atom_range) > 1:
        groups = [lines[i+atom_range[0]:i+atom_range[1]] for i in range(0, len(lines), natom)][:-1]
    else:
        groups = [lines[i+atom_range[0]] for i in range(0, len(lines), natom)][:-1]
    groups = [groups[i][j].split()[1:] for i in range(len(groups)) for j in range(len(groups[i]))]
    return np.array(groups, dtype=np.float64)


def xyz_to_npy(filename):
    """
    function: parse .xyz file to numpy array with float dtype, and cut first column of atom type
    """
    data = xyz_to_groups(filename, cut=2)
    new_data = [[list(map(float, subsublist.split()[1:])) for subsublist in sublist] for sublist in data]
    return np.array(new_data)


def chunk_to_groups(data):
    """
    function: split chunk to single frames
    """
    natom = int(data[0]) + 2
    groups = [data[i:i+natom] for i in range(0, len(data), natom)][:-1]
    return groups


def groups_to_xyz(filename, groups, cut=None):
    """
    function: write groups(some of frame) to .xyz file
    """
    with open(filename, 'w') as f:
        for group in groups:
           for i in group[cut:]:
               f.write(i + '\n')


def group_to_xyz(filename, group, cut=None):
    """
    function: write groups(some of frame) to .xyz file
    """
    with open(filename, 'w') as f:
       for i in group[cut:]:
           f.write(i + '\n')


def coord_to_xyz(filename):
    with open(filename, 'r') as f:
        data = f.read()
    data = data.split('\n')
    data.insert(0, '')
    data.insert(0, str(len(data)-2))
    return data


def atom_mass_parse(group):
    import periodictable
    atom_mass_dict = {}
    names = atom_name_parse()
    for name in names:
        if hasattr(periodictable, name):
            atom = getattr(periodictable, name)
            atom_mass_dict[name] = atom.mass
        else:
            print(f'dont have face named {name}, use mass is 0')
            atom_mass_dict[name] = 0
    return atom_mass_dict


def atom_name_parse(group):
    names = {row.split()[0] for row in group[2:]}
    return names



#!/usr/bin/env python3

import numpy as np
import argparse
import MDAnalysis
from MDAnalysis import Universe
from MDAnalysis.analysis.base import AnalysisBase
from util import cp2k_input_parsing
import warnings
warnings.filterwarnings("ignore")


class Hb_distribution(AnalysisBase):
    def __init__(self, filename, cell, surface, dt=0.001, hb_distance=3.5, hb_angle=35, bin_size=0.2):
        u = Universe(filename)
        u.trajectory.ts.dt = dt
        u.dimensions = cell
        self.u = u
        self.atomgroup = u.select_atoms("all")
        self.hb_distance = hb_distance
        self.hb_angle = hb_angle
        self.bin_size = bin_size
        self.surface = surface
        self.frame_count = 0
        super(Hb_distribution, self).__init__(self.atomgroup.universe.trajectory, verbose=True)

    def _prepare(self):
        bin_num = int(self.u.dimensions[2] / self.bin_size) + 2
        self.accepter = np.zeros(bin_num, dtype=np.float64)
        self.donor = np.zeros(bin_num, dtype=np.float64)

    def _append(self, hb_d, hb_a):
        bins_d = np.floor(hb_d / self.bin_size).astype(int) + 1
        bins_a = np.floor(hb_a / self.bin_size).astype(int) + 1

        bins_d = bins_d[bins_d < len(self.donor)]
        bins_a = bins_a[bins_a < len(self.accepter)]

        np.add.at(self.donor, bins_d, 1)
        np.add.at(self.accepter, bins_a, 1)

        self.frame_count += 1

    def _single_frame(self):
        o_group = self.atomgroup.select_atoms("name O")
        o_pair = MDAnalysis.lib.distances.capped_distance(o_group.positions, o_group.positions, min_cutoff=0, max_cutoff=self.hb_distance, box=self.u.dimensions, return_distances=False)

        o0 = o_group[o_pair[:, 0]]
        o1 = o_group[o_pair[:, 1]]

        o0h1 = self.atomgroup[o0.indices + 1]
        o0h2 = self.atomgroup[o0.indices + 2]
        o1h1 = self.atomgroup[o1.indices + 1]
        o1h2 = self.atomgroup[o1.indices + 2]

        angle_o0h1_o0_o1 = np.degrees(
            MDAnalysis.lib.distances.calc_angles(o0h1.positions, o0.positions, o1.positions, box=self.u.dimensions)
        )
        angle_o0h2_o0_o1 = np.degrees(
            MDAnalysis.lib.distances.calc_angles(o0h2.positions, o0.positions, o1.positions, box=self.u.dimensions)
        )
        angle_o1h1_o1_o0 = np.degrees(
            MDAnalysis.lib.distances.calc_angles(o1h1.positions, o1.positions, o0.positions, box=self.u.dimensions)
        )
        angle_o1h2_o1_o0 = np.degrees(
            MDAnalysis.lib.distances.calc_angles(o1h2.positions, o1.positions, o0.positions, box=self.u.dimensions)
        )

        condition_d = (angle_o0h1_o0_o1 < self.hb_angle) | (angle_o0h2_o0_o1 < self.hb_angle)
        condition_a = (angle_o1h1_o1_o0 < self.hb_angle) | (angle_o1h2_o1_o0 < self.hb_angle)

        hb_d = (o0.positions[:, 2][condition_d] + o1.positions[:, 2][condition_d]) / 2
        hb_a = (o0.positions[:, 2][condition_a] + o1.positions[:, 2][condition_a]) / 2

        self._append(hb_d, hb_a)

    def _conclude(self):
        if self.frame_count > 0:
            average_donor = self.donor / self.frame_count
            average_accepter = self.accepter / self.frame_count
            average_sum = average_donor + average_accepter

        bins_z = np.arange(len(self.donor)) * self.bin_size

        lower_z, upper_z = self.surface
        mask = (bins_z >= lower_z) & (bins_z <= upper_z)
        filtered_bins_z = bins_z[mask] - lower_z
        filtered_average_accepter = average_accepter[mask]
        filtered_average_donor = average_donor[mask]
        filtered_average_sum = average_sum[mask]

        combined_data = np.column_stack((filtered_bins_z, filtered_average_accepter, filtered_average_donor, filtered_average_sum))

        np.savetxt("hb_distribution.dat", combined_data, header="Z\tAccepter\tDonor\tAccepter+Donor", fmt='%.5f', delimiter='\t')


def parse_data(s):
    return [float(x) for x in s.replace(',', ' ').split()]


def parse_r(s):
    return [int(x) for x in s.replace(':', ' ').split()]


def parse_argument():
    parser = argparse.ArgumentParser(description="analysis hb distribution")
    parser.add_argument('filename', type=str, help='filename to analysis')
    parser.add_argument('--cp2k_input_file', type=str, help='input file name of cp2k, default is "input.inp"', default='input.inp')
    parser.add_argument('-r', type=parse_r, help='range of analysis', default=[0, -1, 1])
    parser.add_argument('--cell', type=parse_data, help='set cell, a list of lattice, --cell x,y,z or x,y,z,a,b,c')
    parser.add_argument('--surface', type=parse_data, help='[down_surface_z, up_surface_z]')
    parser.add_argument('--hb_param', type=parse_data, help='[hb_distance, hb_angle], default is [3.5, 35]', default=[3.5, 35])

    return parser.parse_args()


def main():
    args = parse_argument()
    cell = cp2k_input_parsing.get_cell(args.cp2k_input_file, args.cell)

    hb_dist = Hb_distribution(args.filename, cell, args.surface, hb_distance=args.hb_param[0], hb_angle=args.hb_param[1])
    hb_dist.run(start=args.r[0], stop=args.r[1], step=args.r[2])


if __name__ == '__main__':
    main()



"""
output and error for cli
"""

import numpy as np
import sys


def cell_output(cell: list):
    print(f"system cell: x = {cell[0]}, y = {cell[1]}, z = {cell[2]}, a = {cell[3]}\u00B0, b = {cell[4]}\u00B0, c = {cell[5]}\u00B0")


def check_cell(atoms, cell):
    if np.array_equal(atoms.cell.cellpar(), np.array([0., 0., 0., 90., 90., 90.])) and cell is not None:
        atoms.set_cell(cell)
    else:
        raise ValueError("can't parse cell please use --cell set cell")


from ase.io import write, read
import argparse
from ase.geometry import cell_to_cellpar, cellpar_to_cell
import math


def parse_argument():
    parser = argparse.ArgumentParser(description='combain solbox and surface to a interface')
    parser.add_argument('--surface', type=str, help='surface path')
    parser.add_argument('--sol', type=str, help='solbox path')
    parser.add_argument('--move', type=float, help='move at z, default = 0', default=0)
    parser.add_argument('--interval', type=float, help='interval between surface to sol, default is 2', default=2)
    parser.add_argument('--vacuum', type=float, help='vacuum of structure, default is 0', default=0)
    parser.add_argument('--symmetry', help='two side interface, default is false', action='store_true')
    parser.add_argument('--ne', help='two side interface, one side is Ne atom, default is false', action='store_true')
    parser.add_argument('-o', type=str, help='output file name', default='interface')

    return parser.parse_args()


def chformat(input_filename, output_filename, format):
    atoms = read(input_filename)
    write(output_filename, atoms, format=format)


def main():
    args = parse_argument()
    surface = read(args.surface)
    surface.set_pbc(True)
    surface.center()
    sy_surface = surface.copy()
    cell = surface.get_cell()
    [lenx, leny, lenz, anga, angb, angc] = cell_to_cellpar(cell)

    solbox = read(args.sol)
    solbox_cell = solbox.cell.cellpar()
    solbox.set_pbc(True)
    solbox.center()
    tmp_list = solbox.get_positions()
    tmp_list[:, 2] += lenz + args.interval
    solbox.set_positions(tmp_list)

    surface.extend(solbox)
    surface.cell = [lenx, leny, (lenz + args.interval + solbox_cell[2] + args.interval), anga, angb, angc]
    surface.center()

    if args.symmetry:
        tmp_list = surface.get_positions()
        tmp_list[:, 2] += -(lenz + args.interval + solbox_cell[2] + args.interval)
        surface.set_positions(tmp_list)
        surface.extend(sy_surface)
        surface.cell = [lenx, leny, (lenz + args.interval + solbox_cell[2] + args.interval + lenz + args.vacuum), anga, angb, angc]
        surface.center()
    elif args.ne:
        from ase import Atoms
        ne_interval = 4 # adjust water density
        ne_cell = [lenx, leny, 2, 90, 90, 90]
        ne_position = []
        ne_symbols = []
        ne_site = [int(lenx//ne_interval), int(leny//ne_interval)]
        for i in range(ne_site[0]):
            for j in range(ne_site[1]):
                ne_position.append((i*ne_interval, j*ne_interval, 0))
                ne_symbols.append('Ne')
        ne_atoms = Atoms(symbols=ne_symbols, positions=ne_position, cell=ne_cell)
        ne_atoms.center()
        tmp_list = surface.get_positions()
        tmp_list[:, 2] += -(lenz + args.interval + solbox_cell[2])
        surface.set_positions(tmp_list)
        surface.extend(ne_atoms)
        surface.cell = [lenx, leny, (lenz + args.interval + solbox_cell[2] + args.interval + 2 + args.vacuum), anga, angb, angc]
        surface.center()
    else:
        surface.set_pbc(True)
        tmp_list = surface.get_positions()
        #tmp_list[:, 2] -= lenz
        tmp_list[:, 2] -= args.move
        surface.set_positions(tmp_list)


    write(args.o + '.xyz', surface, format='extxyz')
    write(args.o + '.cif', surface, format='cif')
    #chformat(args.o + '.cif', args.o + '.xyz', format='xyz')

if __name__ == '__main__':
    main()



"""
filename: encapsulated_ase.py
function: some encapsulated ase method
"""


from ase.io import iread, read
import io
import numpy as np
from ase.io.cube import read_cube_data
import MDAnalysis


def wrap_to_cell(chunk, cell, name, big=False):
    """
    function: encapsulated ase.Atom.wrap method
    parameter:
        chunk: a list of frame of structure
        cell: a list of cell parameter
        name: path of temp file to write
        big: make 3X3X1 supercell
    return:
        write into a temp file
    """
    # wrap a chunk of .xyz sturcture in a cell, and write wraped structure to file
    if len(chunk) > 1:
        atoms = iread(io.StringIO('\n'.join(chunk)), format='xyz')
    else:
        atoms = iread(chunk, format='xyz')
    with open(name, 'w') as f:
        f.write('')
    for atom in atoms:
        atom.set_cell(cell)
        atom.set_pbc(True)
        atom.wrap()
        if big:
            from ase.build import make_supercell
            P = [ [3, 0, 0], [0, 3, 0], [0, 0, 1] ]
            atom = make_supercell(atom, P)
        atom.write(f'{name}', append='a', format='xyz')


def rdf(chunk, cell, bin_size, name, parallel=True):
    """
    not used
    """
    from ase.geometry.analysis import Analysis
    from math import ceil
    n = ceil(cell[2] / bin_size)
    if parallel:
        atoms = iread(io.StringIO('\n'.join(chunk)), format='xyz')
    else:
        atoms = iread(chunk, format='xyz')
    atom_list = []
    for atom in atoms:
        atom.set_cell(cell)
        atom.set_pbc(True)
        atom_list.append(atom)
    print(len(atom_list))
    ana = Analysis(atom_list)
    rdf = ana.get_rdf(3.5, n, elements=['O'])
    rdf_array = np.vstack(rdf)
    rdf_array = np.mean(rdf_array, axis=0)
    print(name)
    np.save(name, rdf_array)
    ana.clear_cache()


def ave_cube(filepath):
    """
    function: average hartree file in z_coordinate
    parameter:
        filepath: hartree cube file path
    return:
        z_cube_data: a list of cube data alone z axes
        z_coordinates: a list of coordinates of z axes
    """
    # read data from filepath
    data, atoms = read_cube_data(filepath)
    # define need parameter
    npoints = data.shape[2]
    step_size = atoms.cell[2, 2] / ( npoints - 1 )
    # average hartree file, and calculate z_coordinates
    z_coordinates = [i * step_size for i in range(npoints)]
    z_cube_data = data[:, :, :].sum(axis=(0, 1)) / ( data.shape[0] * data.shape[1] )
    return np.column_stack((z_coordinates, z_cube_data))


def atoms_read_with_cell(file_name, cell=None, coord_mode=False, default_cell=np.array([0., 0., 0., 90., 90., 90.])):
    """
    function: read structure file return a Atoms object with cell
    parameter:
        file_name: structure file name
        cell: cell list
        coord_mode: if file name is coord format
        default_cell: default cell of Atoms object
    return:
        atoms: Atoms object with cell
    """
    import sys, os
    from . import cp2k_input_parsing, structure_parsing

    if coord_mode:
        xyz = structure_parsing.coord_to_xyz(file_name)
        atoms = read(io.StringIO('\n'.join(xyz)), format='xyz')
    else:
        atoms = jread(file_name)

    if cell == None:
        cp2k_cell_file_name = ( 'cell.inc', 'setup.inp', 'input.inp' )
        existing_cell_file = [ file for file in cp2k_cell_file_name if os.path.isfile(file) ]
        if existing_cell_file:
            cell = cp2k_input_parsing.parse_cell(existing_cell_file[0])

    if np.array_equal(atoms.cell.cellpar(), default_cell):
        if cell != None:
            atoms.set_cell(cell)
            print('cell: \n' + ', '.join(map(str, atoms.cell.cellpar())))
            return atoms
        else:
            print("no cell")
            sys.exit(1)
    print('cell: \n' + ', '.join(map(str, atoms.cell.cellpar())))
    return atoms


def jread(filepath):
    from ase.io import read
    suffix = filepath.split('.')[-1]
    if suffix == 'data':
        atom = read(filepath, format='lammps-data', atom_style='atomic')
    else:
        atom = read(filepath)

    return atom


def atoms_to_u(atoms):
    virtual_file = io.StringIO()
    cell = atoms.cell.cellpar()
    atoms.write(virtual_file, format='xyz')

    u = MDAnalysis.Universe(virtual_file, format='xyz')
    u.dimensions = cell

    return u


import click, os
from julia import Pkg, Main
from mdkits.util import arg_type
from importlib import resources
import tempfile


@click.command(name="solution")
@click.argument("filename", type=click.Path(exists=True), nargs=-1)
@click.option('--infile', is_flag=True, help="read input mode")
@click.option('--install', is_flag=True, help="install julia and packmol")
@click.option('--water_number', type=int, help="number of water molecules", default=0, show_default=True)
@click.option('-n', type=int, multiple=True, help="number of molecules")
@click.option('--tolerance', type=float, help="tolerance of solution", default=3.5, show_default=True)
@click.option('--cell', type=arg_type.Cell, help="set cell, a list of lattice: --cell x,y,z or x,y,z,a,b,c")
@click.option('--gap', type=float, help="gap between solution and cell", default=1, show_default=True)
def main(filename, infile, install, water_number, n, tolerance, cell, gap):
    """
    build solution model
    """
    if install:
        import julia
        julia.install()
        Pkg.activate("Packmol", shared=True)
        Pkg.add("Packmol")
        Main.exit()

    if cell is None:
        raise ValueError("cell should be provided")

    if len(filename) == 0 and water_number == 0:
        raise ValueError("at least one file should be provided, or water_number should be greater than 0")

    while True:
        try:
            Main.using("Packmol")
            break
        except Exception:
            pass

    if infile:
        for file in filename:
            Main.run_packmol(file)
    else:
        if len(n) != len(filename):
            raise ValueError("number of -n should be equal to number of files")

        temp_file = tempfile.NamedTemporaryFile(mode='w+t', delete=False)
        backslash = "\\"

        structure_input = {}
        output_filenames = []
        if len(filename) > 0:
            for index, file in enumerate(filename):
                structure_input[file] = f"structure {os.path.join(os.getcwd(), file.replace(backslash, '/').replace('./', ''))}\n  number {n[index]}\n  inside box {gap} {gap} {gap} {cell[0]-gap} {cell[1]-gap} {cell[2]-gap}\nend structure\n"

                output_filenames.append(f"{file.replace(backslash, '/').split('.')[-2].split('/')[-1]}_{n[index]}")

        if water_number > 0:
            water_path = resources.files('mdkits.build_cli').joinpath('water.xyz')

            structure_input["water"] = f"structure {water_path}\n  number {water_number}\n  inside box {gap} {gap} {gap} {cell[0]-gap} {cell[1]-gap} {cell[2]-gap}\nend structure\n"

            output_filenames.append(f"{str(water_path).replace(backslash, '/').split('.')[-2].split('/')[-1]}_{water_number}")

        output_filename = "-".join(output_filenames) + ".xyz"
        head_input = f"tolerance {tolerance}\nfiletype xyz\noutput {os.path.join(os.getcwd(), output_filename)}\npbc {cell[0]} {cell[1]} {cell[2]}\n"

        total_input = head_input + "\n".join(structure_input.values())
    
        temp_file.write(total_input)
        temp_file.flush()
        temp_file.close()

        Main.run_packmol(temp_file.name)

        if os.path.exists(temp_file.name):
            os.remove(temp_file.name)

    Main.exit()


if __name__ == "__main__":
    main()


#!/usr/bin/env python3

from ase import io, build
import argparse
from util import encapsulated_ase


def parse_size(s):
    return [int(x) for x in s.replace(',', ' ').split()]

def parse_size1(s):
    return [float(x) for x in s.replace(',', ' ').split()]


def parse_argument():
    parser = argparse.ArgumentParser(description='cut surface of structure')
    parser.add_argument('filename', type=str, help='init structure filename')
    parser.add_argument('--face', type=parse_size, help='face index')
    parser.add_argument('--vacuum', type=float, help='designate vacuum of surface, default is None', default=0.0)
    parser.add_argument('--size', type=parse_size, help='surface size')
    parser.add_argument('--coord', help='coord format', action='store_true')
    parser.add_argument('--cell', type=parse_size1, help='set xyz file cell, --cell x,y,z,a,b,c')

    return parser.parse_args()


def main():
    args = parse_argument()
    atoms = encapsulated_ase.atoms_read_with_cell(args.filename, cell=args.cell, coord_mode=args.coord)
    surface = build.surface(atoms, args.face, args.size[2], vacuum=args.vacuum/2)
    super_cell = [[args.size[0], 0, 0], [0, args.size[1], 0], [0, 0, 1]]
    super_surface = build.make_supercell(surface, super_cell)

    super_surface.write(f"{args.filename.split('.')[-2].split('/')[-1]}_{args.face[0]}{args.face[1]}{args.face[2]}.cif")


if __name__ == '__main__':
    main()



#!/usr/bin/env python3

################################################
# averange cp2k output(or some else file correspond to ase.io.read_cube_data) hartree.cube to z coordinate with python
## file path is need to pay attention
## cycle parameter is need to pay attention
## buck range is need to pay attention
################################################

from numpy import empty, array, mean, append, concatenate
from argparse import ArgumentParser
from util import encapsulated_ase, os_operation


def array_type(string):
	number_list = string.split(',')
	number_array = array(number_list, dtype=float)
	return number_array


def buck_potential(xaxe, potential, range):
	mix = concatenate((xaxe.reshape(-1, 1), potential.reshape(-1, 1)), axis=1)
	mask = (mix[:,0] >= range[0]) & (mix[:,0] <=range[1])
	buck_potential = mix[mask]
	ave_potential = mean(buck_potential[:,1])
	return ave_potential


# set argument
parser = ArgumentParser(description='to handle cp2k output file hartree cube, name should be "hartree-*.cube"')
parser.add_argument('file_name', type=str, nargs='?', help='hartree cube file', default=os_operation.default_file_name('*-v_hartree-1_*.cube', last=True))
parser.add_argument('-b', '--buck_range', type=array_type, help='parameter to calculate mean value of buck', default=None)
parser.add_argument('-o', type=str, help='output file name, default is "out.put"', default='hartree.out')

args = parser.parse_args()


## init output potential file's shape, and define a z axe
init_array = encapsulated_ase.ave_potential(args.file_name)
potential = empty((0, init_array[0].shape[0]))
z_coordinates = array((init_array[1])).reshape(-1, 1)

potential = encapsulated_ase.ave_potential(args.file_name)[0]

aved = mean(potential, axis=0)
total_potential = append(z_coordinates, potential.reshape(-1, 1), axis=1)

## if buck range is exit, out put a difference of potential
if args.buck_range is not None:
	buck_potential = buck_potential(z_coordinates, potential, args.buck_range)
	print(buck_potential)
	with open('hartree_potential.dat', 'w') as f:
		f.write(f"{buck_potential}" + '\n')

## write output
with open(args.o, 'w') as f:
	for value in total_potential:
		f.write(" ".join(map(str, value)) + '\n')




#!/usr/bin/env python3

################################################
# averange cp2k output(or some else file correspond to ase.io.read_cube_data) hartree.cube to z coordinate with python
## file path is need to pay attention
## cycle parameter is need to pay attention
## buck range is need to pay attention
################################################

from ase.io.cube import read_cube_data
import numpy as np
import argparse

def array_type(string):
	number_list = string.split(',')
	number_array = np.array(number_list, dtype=int)
	return number_array


def ave_potential(filepath):
	# is to average hartree file in z_coordinate
	
	## read data from filepath
	data, atoms = read_cube_data(filepath)
	
	## define need parameter
	npoints = data.shape[2]
	step_size = atoms.cell[2, 2] / ( npoints - 1 )
	
	## average hartree file, and calculate z_coordinates
	z_coordinates = [i * step_size for i in range(npoints)]
	z_potential = 27.2114 * data[:, :, :].sum(axis=(0, 1)) / ( data.shape[0] * data.shape[1] )
	return z_potential, z_coordinates


def buck_potential(xaxe, potential, range):
	mix = np.concatenate((xaxe.reshape(-1, 1), potential.reshape(-1, 1)), axis=1)
	mask = (mix[:,0] >= range[0]) & (mix[:,0] <=range[1])
	buck_potential = mix[mask]
	ave_potential = np.mean(buck_potential[:,1])
	return ave_potential


# set argument
parser = argparse.ArgumentParser(description='to handle cp2k output file hartree cube, name should be "hartree-*.cube"')
parser.add_argument('folder_path', type=str, help='folder that contain all hartree cube file')
parser.add_argument('cyc_range', type=array_type, help='cycle parameter, need to seperate with ",", similar with range() -- 1,201  1,201,10')
parser.add_argument('-b', '--buck_range', type=array_type, help='parameter to calculate mean value of buck', default=None)
parser.add_argument('-o', type=str, help='output file name, default is "out.put"', default='hartree.out')

args = parser.parse_args()


## init output potential file's shape, and define a z axe
init_array = ave_potential('{}/hartree-{}.cube'.format(args.folder_path, args.cyc_range[0]))
potential = np.empty((0, init_array[0].shape[0]))
z_coordinates = np.array((init_array[1])).reshape(-1, 1)

## average one hartree file
if len(args.cyc_range) == 3:
	for i in range(args.cyc_range[0], args.cyc_range[1], args.cyc_range[2]):
		file_path = '{}/hartree-{}.cube'.format(args.folder_path, i)
		potential = np.append(potential, [ave_potential(file_path)[0]], axis=0)
else:
	for i in range(args.cyc_range[0], args.cyc_range[1]):
		file_path = '{}/hartree-{}.cube'.format(args.folder_path, i)
		potential = np.append(potential, [ave_potential(file_path)[0]], axis=0)

## average every averaged harterr file, and append to z_coordinates
#aved_potential = potential[:, :].sum(axis=0) / len(range(1, 201))
aved = np.mean(potential, axis=0)
total_potential = np.append(z_coordinates, aved.reshape(-1, 1), axis=1)

## if buck range is exit, out put a difference of potential
if args.buck_range is not None:
	buck_potential = buck_potential(z_coordinates, aved, args.buck_range)
	with open(args.o + 'diff', 'w') as f:
		f.write("{}\t{}\t{}".format(aved[0], buck_potential, aved[0]-buck_potential))

## write output
with open(args.o, 'w') as f:
	for value in total_potential:
		f.write(" ".join(map(str, value)) + '\n')

